Package: semPlot
Type: Package
Title: Path Diagrams and Visual Analysis of Various SEM Packages'
        Output
Version: 1.1.8
Authors@R: c(
  person("Sacha", "Epskamp", email = "mail@sachaepskamp.com",role = c("aut", "cre")),
  person("Simon", "Stuber", role = c("ctb")),
  person("Jason", "Nak", role = c("ctb")),
  person("Myrthe", "Veenman", role = c("ctb")),
  person(given = c("Terrence","D."), family = "Jorgensen", role = c("ctb"), comment = c(ORCID = "0000-0001-5111-6773"))
  )
Maintainer: Sacha Epskamp <mail@sachaepskamp.com>
Depends: R (>= 2.15.0)
Suggests: MplusAutomation (>= 0.5-3)
Imports: qgraph (>= 1.2.4), lavaan (>= 0.5-11), sem (>= 3.1-0), plyr,
        XML, igraph (>= 0.6-3), lisrelToR, rockchalk, colorspace,
        corpcor, methods, OpenMx
ByteCompile: yes
Description: Path diagrams and visual analysis of various SEM packages' output.
URL: https://github.com/SachaEpskamp/semPlot
License: GPL-2
LazyLoad: yes
NeedsCompilation: no
Packaged: 2026-02-11 17:17:37 UTC; sachaepskamp
Author: Sacha Epskamp [aut, cre],
  Simon Stuber [ctb],
  Jason Nak [ctb],
  Myrthe Veenman [ctb],
  Terrence D. Jorgensen [ctb] (ORCID:
    <https://orcid.org/0000-0001-5111-6773>)
Repository: CRAN
Date/Publication: 2026-02-11 18:00:02 UTC
RoxygenNote: 7.3.3
Built: R 4.6.0; ; 2026-04-25 13:36:28 UTC; unix
