
# NEw export:
export(semPlotModel)
S3method(semPlotModel, semspec)
S3method(semPlotModel, regsemplot)
S3method(semPlotModel, semPlotModel)

export(semPaths,semPlotModel,semCors,lisrelModel,ramModel,semSyntax,
       semMatrixAlgebra,modelMatrices,Imin,semPlotModel_lavaanModel)

# export Classes
exportClasses(
              "semPlotModel"
             )

# export Methods
exportMethods(
             "semPlotModel_S4"
           )

S3method("+",semPlotModel)
S3method(semPlotModel,list)
S3method(semPlotModel,lm)
S3method(semPlotModel,principal)
S3method(semPlotModel,princomp)
S3method(semPlotModel,loadings)
S3method(semPlotModel,lisrel)
S3method(semPlotModel,factanal)
S3method(semPlotModel,default)
S3method(semPlotModel,mplus.model)
S3method(semPlotModel,sem)
S3method(semPlotModel,msem)
S3method(semPlotModel,msemObjectiveML)
export(semPlotModel_Onyx)
export(semPlotModel_Amos)
export(exo,"exo<-",endo,"endo<-",lat,"lat<-",man,"man<-")
S3method(semPlotModel,regsem)
S3method(semPlotModel,cvregsem)

# importFrom(MplusAutomation,"readModels")
importFrom(sem,"sem","standardizedCoefficients","specifyModel")
importFrom(lavaan,"lavaan","cfa","standardizedSolution", "standardizedsolution", "parameterEstimates", "parameterestimates","inspect","lavaanNames","lavaanify","lavInspect","lavTech")
importClassesFrom(lavaan,"lavaan")
importFrom(stats,"factanal")
importFrom(rockchalk,standardize)
#importFrom(regsem, "regsem","cv_regsem")
import(plyr)
import(lisrelToR)
import(XML)
import(qgraph)
import(methods)
import(OpenMx)
#import(semTools)
importFrom(igraph,"layout.reingold.tilford","graph.edgelist","shortest.paths")
importFrom(colorspace,rainbow_hcl)
importFrom(corpcor,"pseudoinverse")
 importFrom("grDevices", "col2rgb", "rainbow", "rgb")
  importFrom("graphics", "lines", "par", "text")
  importFrom("stats", "ave", "coef", "cov", "cov2cor", "loadings",
             "median", "pnorm", "weighted.mean")
  importFrom("utils", "packageDescription")
  
  