\name{semPlot-package}
\alias{semPlot-package}
\alias{semPlot}
\docType{package}
\title{
semPlot
}
\description{
Path diagrams and visual analysis of various SEM packages' output. Path diagrams including visualizations of the parameter estimates can be plotted with \code{\link{semPaths}} and visualizations of the implied and observed correlation structures can be plotted using \code{\link{semCors}}. Finally, SEM syntax can be generated using\code{\link{semSyntax}}.

For plotting the graphs the \code{\link[qgraph]{qgraph}} package is used.
}

\author{
Sacha Epskamp (mail@sachaepskamp.com)

Maintainer: Sacha Epskamp <mail@sachaepskamp.com>
}
\references{
github.com/SachaEpskamp/semPlot
}

\keyword{ package }