\name{semPlotModel-edit}
\alias{semPlotModel-edit}
\alias{exo}
\alias{exo<-}
\alias{endo}
\alias{endo<-}
\alias{man}
\alias{man<-}
\alias{lat}
\alias{lat<-}
\title{
Functions to facilitate editting 'semPlotModel' objects.
}
\description{
These functions can be used to easilly call and edit parts of a \code{\link{semPlotModel-class}} object. Currently only manifest/latent and endgenous/exogenous node properties can be set.
}
\usage{
exo(x)
endo(x)
man(x)
lat(x)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
A \code{"semPlotModel"} object
}
}
\author{
Sacha Epskamp <mail@sachaepskamp.com>
}
\seealso{
\code{\link{semPlotModel}}
}
