\name{semPlotModel_S4-methods}
\docType{methods}
\alias{semPlotModel_S4-methods}
\alias{semPlotModel_S4,lavaan-method}
\alias{semPlotModel_S4}
\title{ S4 methods for semPlotModel }
\description{
 S4 generic used only for the \code{\link[lavaan]{lavaan-class}} class. See \code{\link{semPlotModel}} for more information and \code{\link{semPlotModel-class}} for the resulting object.
}
\section{Methods}{
\describe{

\item{\code{signature(object = "lavaan")}}{
A \code{\link[lavaan]{lavaan-class}} object.
}
}}
\keyword{methods}
